{{/* vim: set filetype=mustache: */}}
{{/*
Expand the name of the chart.
*/}}
{{- define "stacklight.name" -}}
{{- default .Chart.Name .Values.nameOverride | trunc 63 | trimSuffix "-" -}}
{{- end -}}

{{/*
Create a default fully qualified app name.
We truncate at 63 chars because some Kubernetes name fields are limited to this (by the DNS naming spec).
If release name contains chart name it will be used as a full name.
*/}}
{{- define "stacklight.fullname" -}}
{{- if .Values.fullnameOverride -}}
{{- .Values.fullnameOverride | trunc 63 | trimSuffix "-" -}}
{{- else -}}
{{- $name := default .Chart.Name .Values.nameOverride -}}
{{- if contains $name .Release.Name -}}
{{- .Release.Name | trunc 63 | trimSuffix "-" -}}
{{- else -}}
{{- printf "%s-%s" .Release.Name $name | trunc 63 | trimSuffix "-" -}}
{{- end -}}
{{- end -}}
{{- end -}}

{{/*
Create chart name and version as used by the chart label.
*/}}
{{- define "stacklight.chart" -}}
{{- printf "%s-%s" .Chart.Name .Chart.Version | replace "+" "_" | trunc 63 | trimSuffix "-" -}}
{{- end -}}

{{/*
Common labels
*/}}
{{- define "stacklight.labels" -}}
app.kubernetes.io/name: {{ include "stacklight.name" . }}
helm.sh/chart: {{ include "stacklight.chart" . }}
app.kubernetes.io/instance: {{ .Release.Name }}
{{- if .Chart.AppVersion }}
app.kubernetes.io/version: {{ .Chart.AppVersion | quote }}
{{- end }}
app.kubernetes.io/managed-by: {{ .Release.Service }}
{{- end -}}

{{/*
Helm Controller high availability ConfigMap name for leader election lock
*/}}
{{- define "stacklight.helmControllerLeaderLock" -}}
stacklight-helm-controller-leader-election
{{- end -}}

{{/*
Resources section
*/}}
{{- define "stacklight.resources" -}}
{{- $resourcesPerCluster := dict }}
{{- if hasKey .inTplValues.resourcesPerClusterSize .service }}
  {{- $resourcesPerCluster = index (index .inTplValues.resourcesPerClusterSize .service) .inTplValues.clusterSize }}
{{- end }}
{{- if .fallbackService }}
  {{- if hasKey .inTplValues.resourcesPerClusterSize .fallbackService }}
    {{- $resourcesPerCluster = mergeOverwrite (index (index .inTplValues.resourcesPerClusterSize .fallbackService) .inTplValues.clusterSize) $resourcesPerCluster }}
  {{- end }}
{{- end }}
{{- $resources := dict }}
{{- if hasKey .inTplValues.resources .service }}
  {{- $resources = index .inTplValues.resources .service }}
{{- end }}
{{- if .fallbackService }}
  {{- if hasKey .inTplValues.resources .fallbackService }}
    {{- $resources = mergeOverwrite (index .inTplValues.resources .fallbackService) $resources }}
  {{- end }}
{{- end }}
{{- $resourcesMerged := mergeOverwrite $resourcesPerCluster $resources }}
{{ dict "resources" $resourcesMerged | toYaml }}
{{- end -}}

{{- define "stacklight.nodeSelector" -}}
{{- $service := .service }}
{{- if not (hasKey .inTplValues.nodeSelector.component $service) }}
  {{- $service = .fallbackService }}
{{- end }}
{{- $nodeSelector := dict }}
{{- $nodeSelector = mergeOverwrite $nodeSelector .inTplValues.nodeSelector.default (index .inTplValues.nodeSelector.component $service | default dict) }}
{{- if $nodeSelector }}
nodeSelector:
{{ toYaml $nodeSelector | indent 2 }}
{{- end }}
{{- end -}}

{{- define "stacklight.tolerations" -}}
{{- $service := .service }}
{{- if not (hasKey .inTplValues.tolerations.component $service) }}
  {{- $service = .fallbackService }}
{{- end }}
{{- $tolerations := concat .inTplValues.tolerations.default (index .inTplValues.tolerations.component $service | default list ) | uniq }}
{{- if $tolerations }}
tolerations:
{{ toYaml $tolerations | indent 2 }}
{{- end }}
{{- end -}}

{{- define "stacklight.alertmanager.route" -}}
{{- $allMatchers := .matchers }}
{{- range $label, $value := .match }}
  {{- $allMatchers = append $allMatchers (printf "%s=\"%s\"" $label $value) }}
{{- end }}
{{- range $label, $value := .match_re }}
  {{- $allMatchers = append $allMatchers (printf "%s=~\"%s\"" $label $value) }}
{{- end }}
{{- $resultRoute := dict }}
{{- with uniq $allMatchers }}
matchers:
  {{- toYaml . | nindent 2 }}
{{- end }}
{{- with .routes }}
routes:
  {{- toYaml . | nindent 2 }}
{{- end }}
{{- end -}}

{{- define "stacklight.commonChartParams" -}}
{{- $artifacts := .root.Files.Get "files/artifacts.yaml" | fromYaml }}
{{- $chartVersion := index (index $artifacts "chartsVersions") .chart }}
{{- $chart := index $chartVersion "chart" }}
{{- $version := index $chartVersion "version" }}
{{- $chartURL := "" }}
{{- if contains "://" $chart -}}
{{- $chartURL = $chart }}
{{- else -}}
{{- $chartURL = print (trimSuffix "/" .root.Values.global.helmBaseUrl) "/" $chart "-" $version ".tgz" }}
{{- end -}}
{{- dict "chartURL" $chartURL "version" $version "namespace" .root.Release.Namespace "helmV3" true | toYaml }}
{{- end -}}

{{- define "stacklight.getImage" -}}
  {{- $artifacts := .root.Files.Get "files/artifacts.yaml" | fromYaml }}
  {{- $images := index $artifacts "dockerImagesVersions" }}
  {{- $image := index $images .image }}
  {{- $registry := .root.Values.global.dockerBaseUrl }}
  {{- $imgSplit := split ":" $image }}
  {{- $repo := list $registry $imgSplit._0 | join "/" }}
  {{- if .flat }}
    {{- list $registry $image | join "/" }}
  {{- else }}
    {{- dict "repository" $repo "tag" $imgSplit._1 | toYaml }}
  {{- end }}
{{- end -}}

{{- define "memUnitEnum" -}}
{{- dict "" 0 "k" 1 "m" 2 "g" 3 | toYaml }}
{{- end -}}

{{- define "normalizeMem" -}}
{{- $mem := index . "memory" | int64 }}
{{- if not (index . "hasI") }}
  {{- $unit := index . "unit" }}
  {{- $unitDict := include "memUnitEnum" . | fromYaml }}
  {{- range $_ := until (index $unitDict $unit | int) }}
    {{- $mem = mul $mem 1000 }}
    {{- $mem = div $mem 1024 }}
  {{- end }}
{{- end }}
{{- $mem }}
{{- end -}}

{{- define "memStringToMap" -}}
{{- $hasI := hasSuffix "i" . }}
{{- $mem := . | trimSuffix "i" | lower }}
{{- $unit := "" }}
{{- range $u := keys (omit (include "memUnitEnum" . | fromYaml) "") }}
  {{- if hasSuffix $u $mem }}
    {{- $unit = $u }}
  {{- end }}
{{- end }}
{{- $mem = $mem | trimSuffix $unit | int64 }}
{{- $memNorm := include "normalizeMem" (dict "memory" $mem "unit" $unit "hasI" $hasI) | int64 }}
{{- dict "memory" $memNorm "unit" $unit | toYaml }}
{{- end -}}

{{- define "memStringToMB" -}}
{{- $memMap := include "memStringToMap" . | fromYaml }}
{{- $mem := index $memMap "memory" }}
{{- $unit := index $memMap "unit" }}
{{- if eq $unit "g" }}
  {{- $mem = mul $mem 1024 }}
{{- end }}
{{- $mem }}
{{- end -}}

{{- define "stacklight.logLevel" -}}
{{- $fallback := .fallbackComponent }}
{{- if not $fallback }}
  {{- $fallback = "dummy" }}
{{- end }}
{{- $key := index (index .root.Values.stacklightLogLevels "component" | default dict) .component | default (index (index .root.Values.stacklightLogLevels "component" | default dict) $fallback) | default (index .root.Values.stacklightLogLevels "default" | default "") -}}
{{- index .logLevels $key | default "" -}}
{{- end -}}

{{- define "stacklight.customAlerts" -}}
{{- $customAlerts := dict }}
{{- range $_, $alert := .Values.prometheusServer.customAlerts }}
  {{- $_ := set $customAlerts $alert.alert (omit $alert "alert") }}
{{- end }}
{{- $customAlerts | toYaml }}
{{- end -}}

{{- define "stacklight.alertsFlags" -}}
{{- $externalEpMonitoringEnabled := not (empty (and .Values.externalEndpointMonitoring.enabled .Values.externalEndpointMonitoring.domains)) }}
{{- $sslCertMonitoringEnabled := not (empty (and .Values.sslCertificateMonitoring.enabled .Values.sslCertificateMonitoring.domains)) }}
alertmanager-webhook-servicenow: {{ .Values.alertmanagerSimpleConfig.serviceNow.enabled }}
alertmanager: {{ not .Values.cortexDev.enabled }}
calico: {{ .Values.calico.enabled }}
cassandra: {{ .Values.tungstenFabricMonitoring.enabled }}
ceph: {{.Values.ceph.enabled }}
docker-swarm-logs: {{ and .Values.mke.enabled .Values.logging.enabled }}
docker-swarm: {{ .Values.mke.enabled }}
etcd: {{ .Values.mke.enabled }}
es-exporters: {{ .Values.logging.enabled }}
external-endpoint: {{ or $sslCertMonitoringEnabled $externalEpMonitoringEnabled }}
fluentd: {{ .Values.logging.enabled }}
general: true
grafana: true
helm-controller: true
ipmi-exporter: {{ .Values.ipmiExporter.enabled }}
ironic: {{ .Values.ironic.enabled }}
kafka: {{ .Values.tungstenFabricMonitoring.enabled }}
kube-prometheus-node-alerting: true
kubernetes-apps: true
kubernetes-resources: true
kubernetes-storage: true
kubernetes-system: true
libvirt: {{ .Values.openstack.enabled }}
mariadb: {{ .Values.openstack.enabled }}
mcc-exporter: {{ .Values.kaasExporterEnabled }}
mcc-release-controller: {{ .Values.mccReleaseControllerMonitoringEnabled }}
memcached: {{ .Values.openstack.enabled }}
mke: {{ .Values.mke.enabled }}
msr: {{ .Values.msrMonitoring.enabled }}
nginx: {{ .Values.nginxExporter.enabled }}
node-network: true
node-time: true
opensearch: {{ .Values.logging.enabled }}
openstack-cinder: {{ .Values.openstack.enabled }}
openstack-general: {{ .Values.openstack.enabled }}
openstack-general-logging: {{ and .Values.openstack.enabled .Values.logging.enabled }}
openstack-ironic: {{ and .Values.openstack.enabled .Values.openstack.ironic.enabled }}
openstack-neutron: {{ and .Values.openstack.enabled (not .Values.tungstenFabricMonitoring.enabled) }}
openstack-nova: {{ .Values.openstack.enabled }}
postgresql-ha: {{ .Values.highAvailabilityEnabled }}
postgresql: true
prometheus: true
prometheus-msteams: {{ .Values.alertmanagerSimpleConfig.msteams.enabled }}
prometheus-relay: {{ not .Values.cortexDev.enabled }}
prometheus-stack: {{ not .Values.cortexDev.enabled }}
pushgateway: {{ .Values.prometheusPushgateway.enabled }}
rabbitmq: {{ or .Values.openstack.enabled .Values.tungstenFabricMonitoring.enabled }}
rabbitmq-operator: {{ .Values.tungstenFabricMonitoring.enabled }}
redis: {{ .Values.tungstenFabricMonitoring.enabled }}
removed: true
sf-notifier: {{ .Values.alertmanagerSimpleConfig.salesForce.enabled }}
{{- $iamTLSEnabled := and .Values.iam.enabled .Values.iam.tls.enabled }}
{{- $telemeterServerTLSEnabled := and .Values.telemetry.telemeterServer.enabled .Values.telemetry.telemeterServer.tls.enabled }}
ssl-mcc: {{ or .Values.mccTLSCertificateMonitoringEnabled $iamTLSEnabled $telemeterServerTLSEnabled }}
{{- $externalFQDNs := or (not (empty .Values.openstack.externalFQDN)) .Values.openstack.externalFQDNs.enabled }}
ssl-openstack: {{ and .Values.openstack.enabled $externalFQDNs }}
ssl: {{ $sslCertMonitoringEnabled }}
stacklight: true
telegraf: {{ or .Values.openstack.enabled .Values.mke.enabled (eq .Values.provider "BareMetal") }}
telegraf-arping-check: {{ and .Values.openstack.enabled (not .Values.tungstenFabricMonitoring.enabled) .Values.openstack.arpingCheck.enabled }}
telegraf-smart: {{ eq .Values.provider "BareMetal" }}
telemeter-client: {{ .Values.telemetry.telemeterClient.enabled }}
telemeter-client-ha: {{ and .Values.telemetry.telemeterClient.enabled .Values.highAvailabilityEnabled }}
telemeter-client-single: {{ and .Values.telemetry.telemeterClient.enabled (not .Values.highAvailabilityEnabled) }}
telemeter-server: {{ .Values.telemetry.telemeterServer.enabled }}
tungsten-fabric: {{ .Values.tungstenFabricMonitoring.enabled }}
tungsten-fabric-logging: {{ and .Values.tungstenFabricMonitoring.enabled .Values.logging.enabled }}
watchdog: {{ .Values.prometheusServer.watchDogAlertEnabled }}
zookeeper: {{ .Values.tungstenFabricMonitoring.enabled }}
{{- end -}}

{{/*
Collect all alerts in single dict
- omit disabled alert groups
- perform overrides
*/}}
{{- define "stacklight.collectAlerts" -}}
{{- $root := . }}
{{- $alertsFlags := include "stacklight.alertsFlags" $root | fromYaml }}
{{- $customAlerts := include "stacklight.customAlerts" $root | fromYaml }}
{{- $resultAlerts := dict }}
{{- range $path, $bytes := .Files.Glob "files/alerts/*.yaml" }}
  {{- $alertfile :=  $root.Files.Get $path | fromYaml }}
  {{- range $group, $groupAlerts := index $alertfile "alerts" }}
    {{- if index $alertsFlags $group }}
      {{- $resultAlerts = merge $resultAlerts $groupAlerts }}
    {{- end }}
  {{- end }}
{{- end }}
{{- $resultAlerts = mergeOverwrite $resultAlerts $customAlerts }}
{{ $resultAlerts | toYaml }}
{{- end -}}

{{- define "stacklight.getEnabledAlerts" -}}
{{- $alertsSieved := dict }}
{{- range $alertName, $alertValues := include "stacklight.collectAlerts" . | fromYaml }}
  {{- if or (not (hasKey $alertValues "enabled")) (index $alertValues "enabled") }}
    {{- $alertConfig := unset (omit $alertValues "_hidden") "enabled" }}
    {{- $_ := set $alertsSieved $alertName $alertConfig }}
  {{- end }}
{{- end }}
{{ $alertsSieved | toYaml }}
{{- end -}}

{{- define "stacklight.getGroupedAlerts" -}}
{{- $alertsGrouped := dict }}
{{- range $alertName, $alertConfig := include "stacklight.getEnabledAlerts" . | fromYaml }}
  {{- $labels := index $alertConfig "labels" }}
  {{- $serviceName := index $labels "service" }}
  {{- if not (hasKey $alertsGrouped $serviceName) }}
    {{- $_ := set $alertsGrouped $serviceName dict }}
  {{- end }}
  {{- $_ := set (index $alertsGrouped $serviceName) $alertName $alertConfig }}
{{- end }}
{{ $alertsGrouped | toYaml }}
{{- end -}}

{{- define "stacklight.collectAllInhibitRules" -}}
{{- $root := . }}
{{- $allInhibitRules := list }}
{{- $resultAlerts := include "stacklight.collectAlerts" $root }}
{{- range $alertname, $alertvalues := $resultAlerts | fromYaml }}
  {{- if or (not (hasKey $alertvalues "enabled")) (index $alertvalues "enabled") }}
    {{- if hasKey $alertvalues "_hidden" }}
      {{- if hasKey $alertvalues._hidden "inhibit_rules" }}
        {{- range $rule := $alertvalues._hidden.inhibit_rules }}

          {{- $source_matchers := list }}
          {{- $hasAlertnameSourceMatcher := false }}
          {{- if hasKey $rule "source_matchers" }}
            {{- range $element := $rule.source_matchers }}
              {{- if hasPrefix "alertname" $element }}
                {{- $hasAlertnameSourceMatcher = true }}
              {{- end }}
            {{- end }}
            {{- $source_matchers = concat $source_matchers $rule.source_matchers }}
          {{- end }}
          {{- if not $hasAlertnameSourceMatcher }}
            {{- $source_matchers = concat (list (printf "%s=%s" "alertname" $alertname)) $source_matchers }}
          {{- end }}
          {{- $_ := set $rule "source_matchers" $source_matchers }}

          {{- $target_matchers := list }}
          {{- $hasAlertnameTargetMatcher := false }}
          {{- if hasKey $rule "target_matchers" }}
            {{- range $element := $rule.target_matchers }}
              {{- if hasPrefix "alertname" $element }}
                {{- $hasAlertnameTargetMatcher = true }}
              {{- end }}
            {{- end }}
            {{- $target_matchers = concat $target_matchers $rule.target_matchers }}
          {{- end }}
          {{- if not $hasAlertnameTargetMatcher }}
            {{- $target_matchers = concat (list (printf "%s=%s" "alertname" $alertname)) $target_matchers }}
          {{- end }}
          {{- $_ := set $rule "target_matchers" $target_matchers }}

          {{- $allInhibitRules = append $allInhibitRules $rule }}
        {{- end }}
      {{- end }}
    {{- end }}
  {{- end }}
{{- end }}
{{- $allInhibitRules | toYaml }}
{{- end -}}

{{- define "stacklight.components" -}}
alerta:
  containers:
  - alerta
  enabled: {{ .Values.alerta.enabled }}
  hardAffinity: {{ .Values.highAvailabilityEnabled }}
  kind: Deployment
  replicas: 2
alertmanager:
  containers:
  - alertmanager
  enabled: {{ not .Values.cortexDev.enabled }}
  hardAffinity: {{ .Values.highAvailabilityEnabled }}
  kind: StatefulSet
  replicas: 2
alertmanagerWebhookServicenow:
  containers:
  - alertmanagerWebhookServicenow
  enabled: {{ .Values.alertmanagerSimpleConfig.serviceNow.enabled }}
  hardAffinity: false
  kind: Deployment
  replicas: 1
blackboxExporter:
  containers:
  - blackboxExporter
  # - configmap-reload
  enabled: true
  hardAffinity: false
  kind: Deployment
  replicas: 1
cerebro:
  containers:
  - cerebro
  enabled: {{ and .Values.logging.enabled .Values.logging.cerebro }}
  hardAffinity: false
  kind: Deployment
  replicas: 1
cortexAlertmanager:
  containers:
  - cortexAlertmanager
  enabled: {{ .Values.cortexDev.enabled }}
  hardAffinity: {{ .Values.highAvailabilityEnabled }}
  kind: StatefulSet
  replicas: {{ if .Values.highAvailabilityEnabled }}{{ if .Values.cortexDev.mimir }}1{{ else }}2{{ end }}{{ else }}1{{ end }}
cortexCompactor:
  containers:
  - cortexCompactor
  enabled: {{ .Values.cortexDev.enabled }}
  hardAffinity: false
  kind: StatefulSet
  replicas: 1
cortexConfigLoader:
  containers:
  - cortexConfigLoader
  enabled: {{ .Values.cortexDev.enabled }}
  kind: Job
cortexDistributor:
  containers:
  - cortexDistributor
  enabled: {{ .Values.cortexDev.enabled }}
  hardAffinity: {{ .Values.highAvailabilityEnabled }}
  kind: Deployment
  replicas: {{ if .Values.highAvailabilityEnabled }}{{ if .Values.cortexDev.mimir }}1{{ else }}2{{ end }}{{ else }}1{{ end }}
cortexIngester:
  containers:
  - cortexIngester
  enabled: {{ .Values.cortexDev.enabled }}
  hardAffinity: {{ .Values.highAvailabilityEnabled }}
  kind: StatefulSet
  replicas: {{ if .Values.highAvailabilityEnabled }}{{ if .Values.cortexDev.mimir }}2{{ else }}3{{ end }}{{ else }}2{{ end }}
cortexNginx:
  containers:
  - cortexNginx
  enabled: {{ .Values.cortexDev.enabled }}
  hardAffinity: {{ .Values.highAvailabilityEnabled }}
  kind: Deployment
  replicas: {{ if .Values.highAvailabilityEnabled }}{{ if .Values.cortexDev.mimir }}1{{ else }}2{{ end }}{{ else }}1{{ end }}
cortexQuerier:
  containers:
  - cortexQuerier
  enabled: {{ .Values.cortexDev.enabled }}
  hardAffinity: {{ .Values.highAvailabilityEnabled }}
  kind: Deployment
  replicas: {{ if .Values.highAvailabilityEnabled }}{{ if .Values.cortexDev.mimir }}1{{ else }}2{{ end }}{{ else }}1{{ end }}
cortexQueryFrontend:
  containers:
  - cortexQueryFrontend
  enabled: {{ .Values.cortexDev.enabled }}
  hardAffinity: {{ .Values.highAvailabilityEnabled }}
  kind: Deployment
  replicas: {{ if .Values.highAvailabilityEnabled }}{{ if .Values.cortexDev.mimir }}1{{ else }}2{{ end }}{{ else }}1{{ end }}
cortexRuler:
  containers:
  - cortexRuler
  enabled: {{ .Values.cortexDev.enabled }}
  hardAffinity: {{ .Values.highAvailabilityEnabled }}
  kind: Deployment
  replicas: {{ if .Values.highAvailabilityEnabled }}{{ if .Values.cortexDev.mimir }}1{{ else }}2{{ end }}{{ else }}1{{ end }}
cortexStoreGateway:
  containers:
  - cortexStoreGateway
  enabled: {{ .Values.cortexDev.enabled }}
  hardAffinity: {{ .Values.highAvailabilityEnabled }}
  kind: StatefulSet
  replicas: {{ if .Values.highAvailabilityEnabled }}{{ if .Values.cortexDev.mimir }}1{{ else }}2{{ end }}{{ else }}1{{ end }}
elasticsearchCurator:
  containers:
  - elasticsearchCurator
  enabled: {{ .Values.logging.enabled }}
  kind: CronJob
elasticsearchExporter:
  containers:
  - elasticsearchExporter
  enabled: {{ .Values.logging.enabled }}
  hardAffinity: false
  kind: Deployment
  replicas: 1
etcd:
  containers:
  - etcd
  enabled: {{ .Values.cortexDev.enabled }}
  hardAffinity: {{ .Values.highAvailabilityEnabled }}
  kind: StatefulSet
  replicas: {{ if .Values.highAvailabilityEnabled }}3{{ else }}1{{ end }}
fluentdLogs:
  containers:
  - fluentdLogs
  enabled: {{ .Values.logging.enabled }}
  kind: DaemonSet
fluentdNotifications:
  containers:
  - fluentdNotifications
  enabled: {{ and .Values.logging.enabled .Values.openstack.enabled }}
  hardAffinity: {{ .Values.highAvailabilityEnabled }}
  kind: Deployment
  replicas: {{ if .Values.highAvailabilityEnabled }}2{{ else }}1{{ end }}
grafana:
  containers:
  - grafana
  {{- if .Values.grafana.renderer.enabled }}
  - grafanaRenderer
  {{- end }}
  enabled: true
  hardAffinity: false
  kind: Deployment
  replicas: 1
helmController:
  containers:
  - stacklightHelmControllerController
  - stacklightHelmControllerTiller
  enabled: {{ .Values.helmController.enabled }}
  hardAffinity: {{ .Values.highAvailabilityEnabled }}
  kind: Deployment
  replicas: 3
iamProxyAlerta:
  containers:
  - iamProxyAlerta
  enabled: {{ and .Values.iam.enabled (not .Values.cortexDev.enabled) .Values.alerta.enabled }}
  hardAffinity: false
  kind: Deployment
  replicas: 1
iamProxyAlertmanager:
  containers:
  - iamProxyAlertmanager
  enabled: {{ and .Values.iam.enabled (not .Values.cortexDev.enabled) }}
  hardAffinity: false
  kind: Deployment
  replicas: 1
iamProxyGrafana:
  containers:
  - iamProxyGrafana
  enabled: {{ .Values.iam.enabled }}
  hardAffinity: false
  kind: Deployment
  replicas: 1
iamProxyOpenSearchDashboards:
  containers:
  - iamProxyOpenSearchDashboards
  enabled: {{ and .Values.iam.enabled .Values.logging.enabled }}
  hardAffinity: false
  kind: Deployment
  replicas: 1
iamProxyPrometheus:
  containers:
  - iamProxyPrometheus
  enabled: {{ .Values.iam.enabled }}
  hardAffinity: false
  kind: Deployment
  replicas: 1
ipmiExporter:
  containers:
  - ipmiExporter
  enabled: {{ .Values.ipmiExporter.enabled }}
  hardAffinity: false
  kind: Deployment
  replicas: 1
kubeStateMetrics:
  containers:
  - kubeStateMetrics
  enabled: true
  hardAffinity: false
  kind: Deployment
  replicas: 1
libvirtExporter:
  containers:
  - libvirtExporter
  enabled: {{ .Values.openstack.enabled }}
  kind: DaemonSet
memcachedBlocks:
  containers:
  - memcachedBlocks
  enabled: {{ and .Values.cortexDev.enabled .Values.cortexDev.memcached.enabled }}
  hardAffinity: {{ .Values.highAvailabilityEnabled }}
  kind: {{ if .Values.highAvailabilityEnabled }}StatefulSet{{ else }}Deployment{{ end }}
  replicas: {{ if .Values.highAvailabilityEnabled }}2{{ else }}1{{ end }}
memcachedBlocksIndex:
  containers:
  - memcachedBlocksIndex
  enabled: {{ and .Values.cortexDev.enabled .Values.cortexDev.memcached.enabled }}
  hardAffinity: {{ .Values.highAvailabilityEnabled }}
  kind: {{ if .Values.highAvailabilityEnabled }}StatefulSet{{ else }}Deployment{{ end }}
  replicas: {{ if .Values.highAvailabilityEnabled }}2{{ else }}1{{ end }}
memcachedBlocksMetadata:
  containers:
  - memcachedBlocksMetadata
  enabled: {{ and .Values.cortexDev.enabled .Values.cortexDev.memcached.enabled }}
  hardAffinity: {{ .Values.highAvailabilityEnabled }}
  kind: {{ if .Values.highAvailabilityEnabled }}StatefulSet{{ else }}Deployment{{ end }}
  replicas: {{ if .Values.highAvailabilityEnabled }}2{{ else }}1{{ end }}
metricCollector:
  containers:
  - metricCollector
  enabled: {{ .Values.telemetry.metricCollector.enabled }}
  hardAffinity: false
  kind: Deployment
  replicas: 1
metricbeat:
  containers:
  - metricbeat
  enabled: {{ .Values.logging.enabled }}
  hardAffinity: false
  kind: Deployment
  replicas: 1
mimirOverridesExporter:
  containers:
  - mimirOverridesExporter
  enabled: {{ and .Values.cortexDev.enabled .Values.cortexDev.mimir }}
  hardAffinity: false
  kind: Deployment
  replicas: 1
minio:
  containers:
  - minio
  enabled: {{ .Values.cortexDev.enabled }}
  hardAffinity: {{ .Values.highAvailabilityEnabled }}
  kind: {{ if .Values.highAvailabilityEnabled }}StatefulSet{{ else }}Deployment{{ end }}
  replicas: {{ if .Values.highAvailabilityEnabled }}3{{ else }}1{{ end }}
minioMakeBucket:
  containers:
  - minioMakeBucket
  enabled: {{ .Values.cortexDev.enabled }}
  kind: Job
minioMakeUser:
  containers:
  - minioMakeUser
  enabled: {{ .Values.cortexDev.enabled }}
  kind: Job
nginxExporter:
  containers:
  - nginxExporter
  enabled: {{ .Values.nginxExporter.enabled }}
  kind: DaemonSet
nodeExporter:
  containers:
  - nodeExporter
  enabled: true
  kind: DaemonSet
opensearch:
  containers:
  - opensearch
  enabled: {{ .Values.logging.enabled }}
  hardAffinity: {{ .Values.highAvailabilityEnabled }}
  kind: StatefulSet
  replicas: {{ if .Values.logging.enabled }}{{ if .Values.highAvailabilityEnabled }}3{{ else }}1{{ end }}{{ else }}0{{ end }}
opensearchDashboards:
  containers:
  - opensearchDashboards
  enabled: {{ .Values.logging.enabled }}
  hardAffinity: false
  kind: Deployment
  replicas: 1
patroni:
  containers:
  - patroniExporter
  - pgsqlExporter
  - postgresql
  enabled: true
  hardAffinity: {{ .Values.highAvailabilityEnabled }}
  kind: StatefulSet
  replicas: {{ if .Values.highAvailabilityEnabled }}3{{ else }}1{{ end }}
prometheusEsExporter:
  containers:
  - prometheusEsExporter
  enabled: {{ .Values.logging.enabled }}
  hardAffinity: false
  kind: Deployment
  replicas: 1
prometheusMsTeams:
  containers:
  - prometheusMsTeams
  enabled: {{ .Values.alertmanagerSimpleConfig.msteams.enabled }}
  hardAffinity: false
  kind: Deployment
  replicas: 1
prometheusRelay:
  containers:
  - prometheusRelay
  enabled: {{ not .Values.cortexDev.enabled }}
  hardAffinity: false
  kind: Deployment
  replicas: 1
prometheusServer:
  containers:
  {{- if .Values.cortexDev.enabled }}
  - cortexPrometheus
  {{- else }}
  - prometheusServer
  {{- end }}
  enabled: true
  hardAffinity: {{ .Values.highAvailabilityEnabled }}
  kind: StatefulSet
  replicas: {{ if .Values.highAvailabilityEnabled }}{{ if and .Values.cortexDev.enabled .Values.cortexDev.mimir }}1{{ else }}2{{ end }}{{ else }}1{{ end }}
pushgateway:
  containers:
  - pushgateway
  enabled: {{ .Values.prometheusPushgateway.enabled }}
  hardAffinity: false
  kind: Deployment
  replicas: 1
sfNotifier:
  containers:
  - sfNotifier
  enabled: {{ .Values.alertmanagerSimpleConfig.salesForce.enabled }}
  hardAffinity: false
  kind: Deployment
  replicas: 1
sfReporter:
  containers:
  - sfReporter
  enabled: {{ .Values.sfReporter.enabled }}
  kind: CronJob
telegrafDockerSwarm:
  containers:
  - telegrafDockerSwarm
  enabled: {{ .Values.mke.enabled }}
  hardAffinity: false
  kind: Deployment
  replicas: 1
telegrafDsArpingCheck:
  containers:
  - telegrafDsArpingCheck
  enabled: {{ and .Values.openstack.enabled (not .Values.tungstenFabricMonitoring.enabled) .Values.openstack.arpingCheck.enabled }}
  kind: DaemonSet
telegrafDsSmart:
  containers:
  - telegrafDsSmart
  enabled: {{ eq .Values.provider "BareMetal" }}
  kind: DaemonSet
telegrafOpenstack:
  containers:
  - telegrafOpenstack
  enabled: {{ .Values.openstack.enabled }}
  hardAffinity: false
  kind: Deployment
  replicas: 1
telemeterClient:
  containers:
  - telemeterClient
  enabled: {{ .Values.telemetry.telemeterClient.enabled }}
  hardAffinity: false
  kind: Deployment
  replicas: 1
telemeterServer:
  containers:
  - telemeterServer
  - telemeterServerAuthServer
  enabled: {{ .Values.telemetry.telemeterServer.enabled }}
  hardAffinity: false
  kind: Deployment
  replicas: 1
tfControllerExporter:
  containers:
  - tfControllerExporter
  enabled: {{ .Values.tungstenFabricMonitoring.enabled }}
  kind: DaemonSet
tfVrouterExporter:
  containers:
  - tfVrouterExporter
  enabled: {{ .Values.tungstenFabricMonitoring.enabled }}
  kind: DaemonSet
{{- end -}}

{{- define "stacklight.replicas" -}}
{{- $replicas := dict }}
{{- range $name, $component := include "stacklight.components" . | fromYaml }}
  {{- $_ := set $replicas $name $component.replicas }}
{{- end }}
{{ toYaml $replicas }}
{{- end -}}

{{- define "stacklight.recordingRules" -}}
groups:
{{- $resultRecordingRules := list }}
{{- range $path, $bytes := .Files.Glob "files/recording-rules/default*.yaml" }}
  {{- $ruleFile := $.Files.Get $path | fromYaml }}
  {{- range $group, $groupRecordingRules := index $ruleFile "recordingRules" }}
    {{- $resultRecordingRules = append $resultRecordingRules $groupRecordingRules }}
  {{- end }}
{{- end }}
{{- if .Values.ceph.enabled }}
  {{- range $path, $bytes := .Files.Glob "files/recording-rules/ceph*.yaml" }}
    {{- $ruleFile := $.Files.Get $path | fromYaml }}
    {{- range $group, $groupRecordingRules := index $ruleFile "recordingRules" }}
      {{- $resultRecordingRules = append $resultRecordingRules $groupRecordingRules }}
    {{- end }}
  {{- end }}
{{- end }}
{{- if .Values.openstack.enabled }}
  {{- range $path, $bytes := .Files.Glob "files/recording-rules/openstack-default*.yaml" }}
    {{- $ruleFile := $.Files.Get $path | fromYaml }}
    {{- range $group, $groupRecordingRules := index $ruleFile "recordingRules" }}
      {{- $resultRecordingRules = append $resultRecordingRules $groupRecordingRules }}
    {{- end }}
  {{- end }}
  {{- if and (not .Values.tungstenFabricMonitoring.enabled) (.Values.openstack.arpingCheck.enabled) }}
    {{- range $path, $bytes := .Files.Glob "files/recording-rules/openstack-nova-instance*.yaml" }}
    {{- $ruleFile := $.Files.Get $path | fromYaml }}
      {{- range $group, $groupRecordingRules := index $ruleFile "recordingRules" }}
        {{- $resultRecordingRules = append $resultRecordingRules $groupRecordingRules }}
      {{- end }}
    {{- end }}
  {{- else }}
    {{- $ruleFile := $.Files.Get "files/recording-rules/openstack-nova-instance.yaml" | fromYaml }}
    {{- range $group, $groupRecordingRules := index $ruleFile "recordingRules" }}
      {{- $resultRecordingRules = append $resultRecordingRules $groupRecordingRules }}
    {{- end }}
  {{- end }}
{{- end }}
  {{ $resultRecordingRules | toYaml | nindent 2 }}
{{- if or .Values.telemetry.telemeterClient.enabled .Values.telemetry.telemeterServer.enabled }}
  {{- $telemeterRules := dict }}
  {{- $relabelingRules := dict }}
  {{- $mcncRules := dict }}
  {{- range $record, $rule := .Files.Get "files/telemeter-recording-rules/general.yaml" | fromYaml }}
    {{- $_ := set $telemeterRules $record $rule.expr }}
    {{- $_ := set $relabelingRules $record $rule.expr }}
    {{- if index $rule "mcnc" }}
      {{- $_ := set $mcncRules $record $rule.expr }}
    {{- end }}
  {{- end }}
  {{- range $record, $rule := .Files.Get "files/telemeter-recording-rules/mcc-release-controller.yaml" | fromYaml }}
    {{- if $.Values.mccReleaseControllerMonitoringEnabled }}
      {{- $_ := set $telemeterRules $record $rule.expr }}
    {{- end }}
    {{- $_ := set $relabelingRules $record $rule.expr }}
    {{- if index $rule "mcnc" }}
      {{- $_ := set $mcncRules $record $rule.expr }}
    {{- end }}
  {{- end }}
  {{- range $record, $rule := .Files.Get "files/telemeter-recording-rules/kaas-exporter.yaml" | fromYaml }}
    {{- if $.Values.kaasExporterEnabled }}
      {{- $_ := set $telemeterRules $record $rule.expr }}
    {{- end }}
    {{- $_ := set $relabelingRules $record $rule.expr }}
    {{- if index $rule "mcnc" }}
      {{- $_ := set $mcncRules $record $rule.expr }}
    {{- end }}
  {{- end }}
  {{- range $record, $rule := .Files.Get "files/telemeter-recording-rules/openstack.yaml" | fromYaml }}
    {{- if $.Values.openstack.enabled }}
      {{- $_ := set $telemeterRules $record $rule.expr }}
    {{- end }}
    {{- $_ := set $relabelingRules $record $rule.expr }}
    {{- if index $rule "mcnc" }}
      {{- $_ := set $mcncRules $record $rule.expr }}
    {{- end }}
  {{- end }}
  {{- range $record, $rule := .Files.Get "files/telemeter-recording-rules/ceph.yaml" | fromYaml }}
    {{- if $.Values.ceph.enabled }}
      {{- $_ := set $telemeterRules $record $rule.expr }}
    {{- end }}
    {{- $_ := set $relabelingRules $record $rule.expr }}
    {{- if index $rule "mcnc" }}
      {{- $_ := set $mcncRules $record $rule.expr }}
    {{- end }}
  {{- end }}
  {{- $id := .Values.clusterId }}
  - name: telemetry.rules
    rules:
  {{- /* a bunch of rules for adding the cluster_id label to the metrics */}}
  {{- range $record, $expr := $telemeterRules }}
    - expr: |
        label_replace({{ $expr }}, "cluster_id", "{{ $id }}", "", "")
      record: telemetry:{{ $record }}
  {{- end }}
  {{- $idKey := .Values.telemetry.clusterIdLabelInjection.key }}
  {{- if and .Values.telemetry.clusterIdLabelInjection.enabled (ne $idKey "cluster_id") }}
    {{- /* a bunch of rules for adding the region_id/management_id label to the metrics */}}
    {{- range $record, $expr := $relabelingRules }}
    - expr: |
      {{- if eq $idKey "management_id" }}
        label_replace(avg(label_replace(telemetry:{{ $record }}{ {{ $idKey }} ="", cluster_id!=""}, "{{ $idKey }}", "{{ $id }}", "", "")) without(_id), "region_id", "{{ $id }}", "region_id", "")
      {{- else }}
        avg(label_replace(telemetry:{{ $record }}{ {{ $idKey }} ="", cluster_id!=""}, "{{ $idKey }}", "{{ $id }}", "", "")) without(_id)
      {{- end }}
      record: telemetry:{{ $record }}
    {{- end }}
  {{- end }}
  {{- if .Values.telemetry.metricCollector.enabled }}
    {{- /* a bunch of rules for preparing the mcnc metrics (in general it is a subset of telemetry metrics) */}}
    {{- range $record, $_ := $mcncRules }}
    - expr: |
        avg(telemetry:{{ $record }}{ {{ $idKey }} !="", cluster_id!=""}) without(_id)
      record: mcnc:{{ $record }}
    {{- end }}
  {{- end }}
{{- end }}
{{- if .Values.prometheusServer.customRecordingRules }}
  {{ toYaml .Values.prometheusServer.customRecordingRules | nindent 2 }}
{{- end }}
{{- end -}}

{{- define "stacklight.logsRetention" -}}
{{- $resultUnit := "days" }}
{{- $resultUnitCount := $ }}
{{- $unitOptions := list "years" "months" "weeks" "days" "hours" }}

{{- range $unit := $unitOptions }}
  {{- $lowerCasedRetention := lower $ }}
  {{- $unitShort := substr 0 1 $unit }}
  {{- if hasSuffix $unitShort $lowerCasedRetention }}
    {{- $resultUnit = $unit }}
    {{- $resultUnitCount = trimSuffix $unitShort $lowerCasedRetention }}
  {{- end }}
{{- end }}

{{- $result := dict "unit" $resultUnit "count" $resultUnitCount }}
{{ $result | toYaml }}
{{- end -}}

{{- define "stacklight.CPUToInt" -}}
{{- $input := toString . }}
{{- $result := 0 }}
{{- if hasSuffix "m" $input }}
  {{- $result = int (trimSuffix "m" $input) }}
{{- else }}
  {{- $result = int (mulf (float64 $input) 1000) }}
{{- end }}
{{- $result }}
{{- end -}}

{{- define "stacklight.memoryToInt" -}}
{{- $input := lower (toString .) }}
{{- $result := 0 }}
{{- $multiplier := 1000 }}
{{- if hasSuffix "i" $input }}
  {{- $multiplier = 1024 }}
  {{- $input = trimSuffix "i" $input }}
{{- end }}
{{- $powers := dict }}
{{- $suffixes := list "k" "m" "g" "t" "p" "e" }}
{{- $currentMultiplier := 1 }}
{{- range $suffix := $suffixes }}
  {{- $currentMultiplier = mul $currentMultiplier $multiplier }}
  {{- $_ := set $powers $suffix $currentMultiplier }}
{{- end }}
{{- range $suf, $power := $powers }}
  {{- if hasSuffix $suf $input }}
    {{- $result = mul (int (trimSuffix $suf $input)) $power }}
  {{- end }}
{{- end }}
{{- $result }}
{{- end -}}

{{- define "stacklight.resourceRequirements" -}}

{{- $cpuLimDS := 0 }}
{{- $cpuReqDS := 0 }}
{{- $memLimDS := 0 }}
{{- $memReqDS := 0 }}

{{- $cpuLimJ := 0 }}
{{- $cpuReqJ := 0 }}
{{- $memLimJ := 0 }}
{{- $memReqJ := 0 }}

{{- $cpuLimW := 0 }}
{{- $cpuReqW := 0 }}
{{- $memLimW := 0 }}
{{- $memReqW := 0 }}

{{- $minNodesNumber := 0 }}

{{- $root := . }}
{{- $components := include "stacklight.components" . | fromYaml }}
{{- $resources := .Values.resources }}

{{- range $name, $component := $components }}
  {{- if $component.enabled }}

    {{- $cpuLim := 0 }}
    {{- $cpuReq := 0 }}
    {{- $memLim := 0 }}
    {{- $memReq := 0 }}

    {{- range $containerName := $component.containers }}

      {{- if not (hasKey $resources $containerName) }}
        {{- $err := printf "%v is missing in .Values.resources\n" $containerName }}
        {{- fail $err }}
      {{- end }}

      {{- $containerResources := include "stacklight.resources" (dict "inTplValues" $root.Values "service" $containerName) | fromYaml }}
      {{- $cpuLim = add $cpuLim (include "stacklight.CPUToInt" (dig "resources" "limits" "cpu" 0 $containerResources)) -}}
      {{- $cpuReq = add $cpuReq (include "stacklight.CPUToInt" (dig "resources" "requests" "cpu" 0 $containerResources)) -}}
      {{- $memLim = add $memLim (include "stacklight.memoryToInt" (dig "resources" "limits" "memory" 0 $containerResources)) -}}
      {{- $memReq = add $memReq (include "stacklight.memoryToInt" (dig "resources" "requests" "memory" 0 $containerResources)) -}}

    {{- end }}

    {{- $kind := $component.kind }}
    {{- if eq $kind "DaemonSet" }}
      {{- $cpuLimDS = add $cpuLimDS $cpuLim }}
      {{- $cpuReqDS = add $cpuReqDS $cpuReq }}
      {{- $memLimDS = add $memLimDS $memLim }}
      {{- $memReqDS = add $memReqDS $memReq }}
    {{- else if or (eq $kind "CronJob") (eq $kind "Job") }}
      {{- $cpuLimJ = add $cpuLimJ $cpuLim }}
      {{- $cpuReqJ = add $cpuReqJ $cpuReq }}
      {{- $memLimJ = add $memLimJ $memLim }}
      {{- $memReqJ = add $memReqJ $memReq }}
      {{- $minNodesNumber = max $minNodesNumber 1 }}
    {{- else if or (eq $kind "Deployment") (eq $kind "StatefulSet") }}
      {{- $replicas := $component.replicas }}
      {{- $cpuLimW = add $cpuLimW (mul $cpuLim $replicas) }}
      {{- $cpuReqW = add $cpuReqW (mul $cpuReq $replicas) }}
      {{- $memLimW = add $memLimW (mul $memLim $replicas) }}
      {{- $memReqW = add $memReqW (mul $memReq $replicas) }}
      {{- if $component.hardAffinity }}
        {{- $minNodesNumber = max $minNodesNumber $replicas }}
      {{- else }}
        {{- $minNodesNumber = max $minNodesNumber 1 }}
      {{- end }}
    {{- else }}
      {{- $err := printf "unexpected kind value %v of %v component\n" $kind $name }}
      {{- fail $err }}
    {{- end }}

  {{- end }}
{{- end }}

{{- printf "DaemonSets (required on every node in the cluster):\n" -}}
{{- printf "  CPU:\n    req: %vm\n    lim: %vm\n  MEM:\n    req: %vMi\n    lim: %vMi\n" $cpuReqDS $cpuLimDS (div $memReqDS 1048576) (div $memLimDS 1048576) -}}
{{- printf "CronJobs/Jobs (required sporadically on StackLight nodes):\n" -}}
{{- printf "  CPU:\n    req: %vm\n    lim: %vm\n  MEM:\n    req: %vMi\n    lim: %vMi\n" $cpuReqJ $cpuLimJ (div $memReqJ 1048576) (div $memLimJ 1048576) -}}
{{- printf "Deployments/StatefulSets (required constantly on StackLight nodes):\n" -}}
{{- printf "  CPU:\n    req: %vm\n    lim: %vm\n  MEM:\n    req: %vMi\n    lim: %vMi\n" $cpuReqW $cpuLimW (div $memReqW 1048576) (div $memLimW 1048576) -}}
{{- printf "Minimum StackLight nodes required: %v\n" $minNodesNumber -}}

{{- end -}}

{{- define "stacklight.addClusterId" -}}
{{- if not (empty .root.Values.clusterId) }}
  {{- $_ := set .alert.labels "cluster_id" .root.Values.clusterId }}
{{- end }}
{{- .alert | toYaml }}
{{- end -}}

{{- define "ipmiExporter.hostsToModules" -}}
{{- $moduleToAddress := dict }}
{{- if kindIs "map" . }}
  {{- range $namespace, $hosts := . }}
    {{- range $name, $address := $hosts }}
      {{- $_ := set $moduleToAddress (list $namespace $name | join "_") $address }}
    {{- end }}
  {{- end }}
{{- else }}
  {{- range $hostConfig := . }}
    {{- $_ := set $moduleToAddress (list $hostConfig.namespace $hostConfig.name | join "_") $hostConfig.address }}
  {{- end }}
{{- end }}
{{- $moduleToAddress | toYaml }}
{{- end -}}
